@@ -2,8 +2,9 @@
 """
 besapi.py
 
-Created by Matt Hansen (mah60@psu.edu) on 2014-03-20.
-Enhancements by James Stewart since 2021
+MIT License
+Copyright (c) 2014 Matt Hansen
+Maintained by James Stewart since 2021
 
 Library for communicating with the BES (BigFix) REST API.
 """
@@ -88,6 +89,30 @@ def elem2dict(node):
     return result
 
 
+# https://stackoverflow.com/questions/16159969/replace-all-text-between-2-strings-python
+def replace_text_between(
+    original_text, first_delimiter, second_delimiter, replacement_text
+):
+    """Replace text between delimeters. Each delimiter should only appear once."""
+    leading_text = original_text.split(first_delimiter)[0]
+    trailing_text = original_text.split(second_delimiter)[1]
+
+    return (
+        leading_text
+        + first_delimiter
+        + replacement_text
+        + second_delimiter
+        + trailing_text
+    )
+
+
+# https://github.com/jgstew/generate_bes_from_template/blob/bcc6c79632dd375c2861608ded3ae5872801a669/src/generate_bes_from_template/generate_bes_from_template.py#L87-L92
+def parse_bes_modtime(string_datetime):
+    """parse datetime string to object"""
+    # ("%a, %d %b %Y %H:%M:%S %z")
+    return datetime.datetime.strptime(string_datetime, "%a, %d %b %Y %H:%M:%S %z")
+
+
 # # https://docs.python-requests.org/en/latest/user/advanced/#transport-adapters
 # class HTTPAdapterBiggerBlocksize(requests.adapters.HTTPAdapter):
 #     """custom HTTPAdapter for requests to override blocksize
@@ -133,6 +158,8 @@ class BESConnection:
         if not verify:
             # disable SSL warnings
             requests.packages.urllib3.disable_warnings()  # pylint: disable=no-member
+        self.verify = verify
+        self.last_connected = None
 
         self.username = username
         self.session = requests.Session()
@@ -151,8 +178,12 @@ class BESConnection:
             rootserver = rootserver + ":52311"
 
         self.rootserver = rootserver
-        self.verify = verify
-        self.last_connected = None
+        try:
+            # get root server port
+            self.rootserver_port = int(rootserver.split("://", 1)[1].split(":", 1)[1])
+        except BaseException:
+            # if error, assume default
+            self.rootserver_port = 52311
 
         self.login()
 
@@ -190,9 +221,6 @@ class BESConnection:
 
     def get(self, path="help", **kwargs):
         """HTTP GET request"""
-        # avoid infinite loop:
-        if "login" not in path:
-            self.login()
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.get(self.url(path), verify=self.verify, **kwargs)
@@ -200,7 +228,6 @@ class BESConnection:
 
     def post(self, path, data, **kwargs):
         """HTTP POST request"""
-        self.login()
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.post(self.url(path), data=data, verify=self.verify, **kwargs)
@@ -208,7 +235,6 @@ class BESConnection:
 
     def put(self, path, data, **kwargs):
         """HTTP PUT request"""
-        self.login()
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.put(self.url(path), data=data, verify=self.verify, **kwargs)
@@ -216,7 +242,6 @@ class BESConnection:
 
     def delete(self, path, **kwargs):
         """HTTP DELETE request"""
-        self.login()
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.delete(self.url(path), verify=self.verify, **kwargs)
@@ -224,7 +249,6 @@ class BESConnection:
 
     def session_relevance_xml(self, relevance, **kwargs):
         """Get Session Relevance Results XML"""
-        self.login()
         self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.post(
@@ -241,15 +265,20 @@ class BESConnection:
         # print(rel_result)
         result = []
         try:
-            for item in rel_result.objectify_text(rel_result.text).Query.Result.Answer:
+            for item in rel_result.besobj.Query.Result.Answer:
                 result.append(item.text)
         except AttributeError as err:
             # print(err)
             if "no such child: Answer" in str(err):
-                result.append(
-                    "ERROR: "
-                    + rel_result.objectify_text(rel_result.text).Query.Error.text
-                )
+                try:
+                    result.append("ERROR: " + rel_result.besobj.Query.Error.text)
+                except AttributeError as err:
+                    if "no such child: Error" in str(err):
+                        result.append("<Nothing> Nothing returned, but no error.")
+                        besapi_logger.info("Query did not return any results")
+                    else:
+                        besapi_logger.error("%s\n%s", err, rel_result.text)
+                        raise
             else:
                 raise
         return result
@@ -269,6 +298,9 @@ class BESConnection:
                 self.last_connected,
                 duration_minutes,
             )
+            # default timeout is 5 minutes
+            # I'm not sure if this is required
+            # or if 'requests' would handle this automatically anyway
             if int(duration_minutes) > 3:
                 besapi_logger.info("Refreshing Login to prevent timeout.")
                 self.last_connected = None
@@ -449,6 +481,29 @@ class BESConnection:
         with open(file_path, "rb") as f:
             return self.post(self.url("upload"), data=f, headers=headers)
 
+    def parse_upload_result_to_prefetch(
+        self, result_upload, use_localhost=True, use_https=True
+    ):
+        """take a rest response from an upload and parse into prefetch"""
+        file_url = str(result_upload.besobj.FileUpload.URL)
+        if use_https:
+            file_url = file_url.replace("http://", "https://")
+        # there are 3 different posibilities for the server FQDN
+        # localhost
+        # self.rootserver (without port number)
+        # the returned value from the upload result
+        if use_localhost:
+            file_url = replace_text_between(
+                file_url, "://", ":" + str(self.rootserver_port), "localhost"
+            )
+
+        # get tail of `Name` in FileUpload Result
+        file_name = str(result_upload.besobj.FileUpload.Name).rsplit("/", 1)[-1]
+        file_size = int(result_upload.besobj.FileUpload.Size)
+        file_sha1 = result_upload.besobj.FileUpload.SHA1
+        file_sha256 = result_upload.besobj.FileUpload.SHA256
+        return f"prefetch {file_name} sha1:{file_sha1} size:{file_size} {file_url} sha256:{file_sha256}"
+
     def get_content_by_resource(self, resource_url):
         """get a single content item by resource"""
         # Get Specific Content
@@ -467,6 +522,100 @@ class BESConnection:
         #     site_path = resource_url.split("/")[-3] + "/" + site_name
         return content
 
+    def update_item_from_file(self, file_path, site_path=None):
+        """update an item by name and last modified"""
+        site_path = self.get_current_site_path(site_path)
+        bes_tree = etree.parse(file_path)
+        # get name of first child tag of BES
+        # - https://stackoverflow.com/a/3601919/861745
+        bes_type = str(bes_tree.xpath("name(/BES/*[1])"))
+        bes_title = bes_tree.xpath("/BES/*[1]/Title/text()")[0]
+        # get last modification time:
+        bes_last_mod = bes_tree.xpath(
+            '/BES/*[1]/MIMEField[Name[contains(text(), "x-fixlet-modification-time")]]/Value/text()'
+        )[0]
+        bes_last_mod_obj = parse_bes_modtime(bes_last_mod)
+
+        print(bes_title)
+        print(bes_type)
+        print(bes_last_mod)
+        print(bes_last_mod_obj)
+
+        return "WORK IN PROGRESS: besapi.update_item_from_file()"
+
+    def save_item_to_besfile(
+        self,
+        xml_string,
+        export_folder="./",
+        name_trim=100,
+    ):
+        """save an xml string to bes file"""
+        item_folder = export_folder
+        if not os.path.exists(item_folder):
+            os.makedirs(item_folder)
+
+        content_obj = RESTResult.objectify_text(None, xml_string)
+        # get first tag in XML that is the Type
+        content_type_tag = list(content_obj.__dict__.keys())[0]
+        item = content_obj[content_type_tag]
+        item_path = item_folder + "/%s.bes" % sanitize_txt(
+            item.Title.text[:name_trim],
+        )
+        item_path = item_path.replace("//", "/")
+        with open(
+            item_path,
+            "wb",
+        ) as bes_file:
+            bes_file.write(xml_string.encode("utf-8"))
+        return item_path
+
+    def export_item_by_resource(
+        self,
+        content_resource,
+        export_folder="./",
+        name_trim=100,
+        include_item_type_folder=False,
+        include_item_id=False,
+    ):
+        """export a single item by resource
+        example resources:
+         - content_type/site_type/site/id
+         - https://localhost:52311/api/content_type/site_type/site/id
+        """
+
+        # Get Specific Content
+        content = self.get_content_by_resource(content_resource)
+        if not content:
+            besapi_logger.warning("Content not found")
+            return None
+
+        # get first tag in XML that is the Type
+        content_type_tag = list(content.besobj.__dict__.keys())[0]
+        item_id = int(content_resource.split("/")[-1])
+        item = content.besobj[content_type_tag]
+        # print(item.__dict__.keys())
+        item_folder = export_folder
+        if include_item_type_folder:
+            item_folder = export_folder + "%s" % sanitize_txt(content_type_tag)
+        # print(item_folder)
+        if not os.path.exists(item_folder):
+            os.makedirs(item_folder)
+        item_path = item_folder + "/%s.bes" % sanitize_txt(
+            item.Title.text[:name_trim],
+        )
+        if include_item_id:
+            item_path = item_folder + "/%s-%s.bes" % sanitize_txt(
+                item_id,
+                item.Title.text[:name_trim],
+            )
+        item_path = item_path.replace("//", "/")
+        with open(
+            item_path,
+            "wb",
+        ) as bes_file:
+            bes_file.write(content.text.encode("utf-8"))
+        return item_path
+
     def export_site_contents(
         self,
         site_path=None,
