@@ -2,7 +2,9 @@
 """
 bescli.py
 
-Created by Matt Hansen (mah60@psu.edu) on 2014-07-21.
+MIT License
+Copyright (c) 2014 Matt Hansen
+Maintained by James Stewart since 2021
 
 Simple command line interface for the BES (BigFix) REST API.
 """
@@ -48,6 +50,16 @@ class BESCLInterface(Cmd):
 
     def do_get(self, line):
         """Perform get request to BigFix server using provided api endpoint argument"""
+
+        # Remove root server prefix:
+        # if root server prefix is not removed
+        # and root server is given as IP Address,
+        # then `robjs` will not work
+        if "/api/" in line:
+            line = str(line).split("/api/", 1)[1]
+            self.pfeedback("get " + line)
+
+        # allow use of `get resource/path.obj_attribute.attribute`
         robjs = line.split(".")
 
         if self.bes_conn:
@@ -65,6 +77,11 @@ class BESCLInterface(Cmd):
         else:
             self.pfeedback("Not currently logged in. Type 'login'.")
 
+    def do_post(self, statement):
+        """post file as data to path"""
+        print(statement)
+        print("not yet implemented")
+
     def do_config(self, conf_file=None):
         """Attempt to load config info from file and login"""
         self.do_conf(conf_file)
@@ -293,14 +310,25 @@ class BESCLInterface(Cmd):
             if statement.raw:
                 # get everything after `query `
                 rel_text = statement.raw.split(" ", 1)[1]
-                self.poutput(f"Q: {rel_text}")
+                self.pfeedback(f"Q: {rel_text}")
                 rel_result = self.bes_conn.session_relevance_string(rel_text)
-                self.poutput(f"A: {rel_result}")
+                self.pfeedback("A: ")
+                self.poutput(rel_result)
 
     def do_version(self, statement=None):
         """output version of besapi"""
         self.poutput(f"besapi version: {__version__}")
 
+    def do_get_action(self, statement=None):
+        """usage: get_action 123"""
+        result_op = self.bes_conn.get(f"action/{statement}")
+        self.poutput(result_op)
+
+    def do_get_operator(self, statement=None):
+        """usage: get_operator ExampleOperatorName"""
+        result_op = self.bes_conn.get_user(statement)
+        self.poutput(result_op)
+
     def do_get_current_site(self, statement=None):
         """output current site path context"""
         self.poutput(
@@ -317,6 +345,10 @@ class BESCLInterface(Cmd):
         """get a specific item by resource url"""
         print(self.bes_conn.get_content_by_resource(resource_url))
 
+    def do_export_item_by_resource(self, statement):
+        """export content itemb to current folder"""
+        print(self.bes_conn.export_item_by_resource(statement))
+
     def do_export_site(self, site_path):
         """export site contents to current folder"""
         self.bes_conn.export_site_contents(
@@ -334,7 +366,9 @@ class BESCLInterface(Cmd):
         if not os.access(file_path, os.R_OK):
             print(file_path, "is not a readable file")
         else:
-            print(self.bes_conn.upload(file_path))
+            upload_result = self.bes_conn.upload(file_path)
+            print(upload_result)
+            print(self.bes_conn.parse_upload_result_to_prefetch(upload_result))
 
     complete_create_group = Cmd.path_complete
 
@@ -363,6 +397,15 @@ class BESCLInterface(Cmd):
         else:
             print(self.bes_conn.create_site_from_file(file_path))
 
+    complete_update_item = Cmd.path_complete
+
+    def do_update_item(self, file_path):
+        """update bigfix content item from bes file"""
+        if not os.access(file_path, os.R_OK):
+            print(file_path, "is not a readable file")
+        else:
+            print(self.bes_conn.update_item_from_file(file_path))
+
 
 def main():
     """Run the command loop if invoked"""
